diff -ru /home/aystarik/reference/linux-2.6.12/drivers/acpi/dispatcher/dsmethod.c ./drivers/acpi/dispatcher/dsmethod.c
--- /home/aystarik/reference/linux-2.6.12/drivers/acpi/dispatcher/dsmethod.c	2005-08-25 13:53:50.000000000 +0400
+++ ./drivers/acpi/dispatcher/dsmethod.c	2005-08-26 23:21:14.000000000 +0400
@@ -243,6 +243,15 @@
 		status = acpi_ex_system_wait_semaphore (obj_desc->method.semaphore,
 				 ACPI_WAIT_FOREVER);
 	}
+	/*
+	 * allocate owner id for this method
+	*/
+	if (!obj_desc->method.thread_count) {
+		status = acpi_ut_allocate_owner_id (&obj_desc->method.owner_id);
+		if (ACPI_FAILURE (status)) {
+			return_ACPI_STATUS (status);
+		}
+	}
 
 	/*
 	 * Increment the method parse tree thread count since it has been
@@ -299,11 +308,6 @@
 		return_ACPI_STATUS (AE_NULL_OBJECT);
 	}
 
-	status = acpi_ut_allocate_owner_id (&obj_desc->method.owner_id);
-	if (ACPI_FAILURE (status)) {
-		return_ACPI_STATUS (status);
-	}
-
 	/* Init for new method, wait on concurrency semaphore */
 
 	status = acpi_ds_begin_method_execution (method_node, obj_desc,
@@ -387,23 +391,17 @@
 
 	if (obj_desc->method.method_flags & AML_METHOD_INTERNAL_ONLY) {
 		status = obj_desc->method.implementation (next_walk_state);
-		return_ACPI_STATUS (status);
 	}
-
-	return_ACPI_STATUS (AE_OK);
-
-
-	/* On error, we must delete the new walk state */
-
+	goto end;
 cleanup:
-	acpi_ut_release_owner_id (&obj_desc->method.owner_id);
+	/* Decrement the thread count on the method parse tree */
 	if (next_walk_state && (next_walk_state->method_desc)) {
-		/* Decrement the thread count on the method parse tree */
-
-	   next_walk_state->method_desc->method.thread_count--;
+		next_walk_state->method_desc->method.thread_count--;
 	}
-	(void) acpi_ds_terminate_control_method (next_walk_state);
+	/* On error, we must delete the new walk state */
+	acpi_ds_terminate_control_method (next_walk_state);
 	acpi_ds_delete_walk_state (next_walk_state);
+end:
 	return_ACPI_STATUS (status);
 }
 
@@ -491,7 +489,7 @@
  *
  * PARAMETERS:  walk_state          - State of the method
  *
- * RETURN:      Status
+ * RETURN:      None
  *
  * DESCRIPTION: Terminate a control method.  Delete everything that the method
  *              created, delete all locals and arguments, and delete the parse
@@ -499,7 +497,7 @@
  *
  ******************************************************************************/
 
-acpi_status
+void
 acpi_ds_terminate_control_method (
 	struct acpi_walk_state          *walk_state)
 {
@@ -512,14 +510,14 @@
 
 
 	if (!walk_state) {
-		return (AE_BAD_PARAMETER);
+		return_VOID;
 	}
 
 	/* The current method object was saved in the walk state */
 
 	obj_desc = walk_state->method_desc;
 	if (!obj_desc) {
-		return_ACPI_STATUS (AE_OK);
+		return_VOID;
 	}
 
 	/* Delete all arguments and locals */
@@ -533,7 +531,7 @@
 	 */
 	status = acpi_ut_acquire_mutex (ACPI_MTX_PARSER);
 	if (ACPI_FAILURE (status)) {
-		return_ACPI_STATUS (status);
+		return_VOID;
 	}
 
 	/* Signal completion of the execution of this method if necessary */
@@ -586,7 +584,7 @@
 		 */
 		status = acpi_ut_acquire_mutex (ACPI_MTX_NAMESPACE);
 		if (ACPI_FAILURE (status)) {
-			return_ACPI_STATUS (status);
+			goto cleanup;
 		}
 
 		if (method_node->child) {
@@ -602,12 +600,11 @@
 		acpi_ut_release_owner_id (&walk_state->method_desc->method.owner_id);
 
 		if (ACPI_FAILURE (status)) {
-			return_ACPI_STATUS (status);
+			goto cleanup;
 		}
 	}
-
-	status = acpi_ut_release_mutex (ACPI_MTX_PARSER);
-	return_ACPI_STATUS (status);
+cleanup:
+	acpi_ut_release_mutex (ACPI_MTX_PARSER);
 }
 
 
diff -ru /home/aystarik/reference/linux-2.6.12/drivers/acpi/parser/psparse.c ./drivers/acpi/parser/psparse.c
--- /home/aystarik/reference/linux-2.6.12/drivers/acpi/parser/psparse.c	2005-08-25 13:53:50.000000000 +0400
+++ ./drivers/acpi/parser/psparse.c	2005-08-26 23:40:25.000000000 +0400
@@ -437,7 +437,6 @@
 	struct acpi_walk_state          *walk_state)
 {
 	acpi_status                     status;
-	acpi_status                     terminate_status;
 	struct acpi_thread_state        *thread;
 	struct acpi_thread_state        *prev_walk_list = acpi_gbl_current_walk_list;
 	struct acpi_walk_state          *previous_walk_state;
@@ -507,7 +506,10 @@
 		else if ((status != AE_OK) && (walk_state->method_desc)) {
 			ACPI_REPORT_METHOD_ERROR ("Method execution failed",
 				walk_state->method_node, NULL, status);
-
+			
+			/* Make sure that failed method will be cleaned as if it was executed */
+			walk_state->parse_flags |= ACPI_PARSE_EXECUTE;
+			
 			/* Check for possible multi-thread reentrancy problem */
 
 			if ((status == AE_ALREADY_EXISTS) &&
@@ -523,14 +525,6 @@
 			}
 		}
 
-		if (walk_state->method_desc) {
-			/* Decrement the thread count on the method parse tree */
-
-			if (walk_state->method_desc->method.thread_count) {
-				walk_state->method_desc->method.thread_count--;
-			}
-		}
-
 		/* We are done with this walk, move on to the parent if any */
 
 		walk_state = acpi_ds_pop_walk_state (thread);
@@ -544,13 +538,11 @@
 		 * there's lots of cleanup to do
 		 */
 		if ((walk_state->parse_flags & ACPI_PARSE_MODE_MASK) == ACPI_PARSE_EXECUTE) {
-			terminate_status = acpi_ds_terminate_control_method (walk_state);
-			if (ACPI_FAILURE (terminate_status)) {
-				ACPI_REPORT_ERROR ((
-					"Could not terminate control method properly\n"));
-
-				/* Ignore error and continue */
+			/* Decrement the thread count on the method parse tree */
+			if (walk_state->method_desc) {
+				walk_state->method_desc->method.thread_count--;
 			}
+			acpi_ds_terminate_control_method (walk_state);
 		}
 
 		/* Delete this walk state and all linked control states */
diff -ru /home/aystarik/reference/linux-2.6.12/drivers/acpi/parser/psxface.c ./drivers/acpi/parser/psxface.c
--- /home/aystarik/reference/linux-2.6.12/drivers/acpi/parser/psxface.c	2005-08-25 13:53:50.000000000 +0400
+++ ./drivers/acpi/parser/psxface.c	2005-08-26 23:40:28.000000000 +0400
@@ -110,16 +110,6 @@
 	}
 
 	/*
-	 * Get a new owner_id for objects created by this method. Namespace
-	 * objects (such as Operation Regions) can be created during the
-	 * first pass parse.
-	 */
-	status = acpi_ut_allocate_owner_id (&info->obj_desc->method.owner_id);
-	if (ACPI_FAILURE (status)) {
-		return_ACPI_STATUS (status);
-	}
-
-	/*
 	 * The caller "owns" the parameters, so give each one an extra
 	 * reference
 	 */
@@ -151,10 +141,6 @@
 
 
 cleanup:
-	if (info->obj_desc->method.owner_id) {
-		acpi_ut_release_owner_id (&info->obj_desc->method.owner_id);
-	}
-
 	/* Take away the extra reference that we gave the parameters above */
 
 	acpi_ps_update_parameter_list (info, REF_DECREMENT);
diff -ru /home/aystarik/reference/linux-2.6.12/drivers/acpi/utilities/utmisc.c ./drivers/acpi/utilities/utmisc.c
--- /home/aystarik/reference/linux-2.6.12/drivers/acpi/utilities/utmisc.c	2005-08-25 13:53:50.000000000 +0400
+++ ./drivers/acpi/utilities/utmisc.c	2005-08-26 23:40:57.000000000 +0400
@@ -74,7 +74,8 @@
 
 	ACPI_FUNCTION_TRACE ("ut_allocate_owner_id");
 
-
+	WARN_ON(*owner_id);
+	
 	/* Mutex for the global ID mask */
 
 	status = acpi_ut_acquire_mutex (ACPI_MTX_CACHES);
diff -ru /home/aystarik/reference/linux-2.6.12/include/acpi/acdispat.h ./include/acpi/acdispat.h
--- /home/aystarik/reference/linux-2.6.12/include/acpi/acdispat.h	2005-08-25 13:41:16.000000000 +0400
+++ ./include/acpi/acdispat.h	2005-08-26 23:09:28.000000000 +0400
@@ -249,7 +249,7 @@
 	struct acpi_walk_state          *walk_state,
 	union acpi_operand_object       *return_desc);
 
-acpi_status
+void
 acpi_ds_terminate_control_method (
 	struct acpi_walk_state          *walk_state);
 
